## This R script that calculates the reference numbers 
## and calculates the change rate over distribution versions.
##
## Author: Samu Varjonen <samu.varjonen ( a ) hiit.fi>

## Function to do the calculations per csv
calculate_refs <- function(dapper, hardy, lucid, precise, rout){

  ##PREPARE DATA FOR DAPPER
  ## remove the app_name column
  ddapper <- read.csv(dapper)
  ddapper <- ddapper[, -1]
  ## remove the category column
  ddapper <- ddapper[, -1]
  
  ##PREPARE DATA FOR HARDY
  ## remove the app_name column
  dhardy <- read.csv(hardy)
  dhardy <- dhardy[, -1]
  ## remove the category column
  dhardy <- dhardy[, -1]

  ##PREPARE DATA FOR LUCID
  ## remove the app_name column
  dlucid <- read.csv(lucid)
  dlucid <- dlucid[, -1]
  ## remove the category column
  dlucid <- dlucid[, -1]

  ##PREPARE DATA FOR PRECISE
  ## remove the app_name column
  dprecise <- read.csv(precise)
  dprecise <- dprecise[, -1]
  ## remove the category column
  dprecise <- dprecise[, -1]

  ##REFNUMS FOR DAPPER
  ## calculate the column sums
  column_sums <- apply(ddapper, 2, sum)
  ## how many lines there is in the csv
  number_of_rows <- nrow(ddapper)
  ## do the reference numbers, times 100 to get the decimal in place
  dapper_reference_numbers <- (column_sums / number_of_rows) * 100

  ##REFNUMS FOR HARDY
  ## calculate the column sums
  column_sums <- apply(dhardy, 2, sum)
  ## how many lines there is in the csv
  number_of_rows <- nrow(dhardy)
  ## do the reference numbers, times 100 to get the decimal in place
  hardy_reference_numbers <- (column_sums / number_of_rows) * 100

  ##REFNUMS FOR LUCID
  ## calculate the column sums
  column_sums <- apply(dlucid, 2, sum)
  ## how many lines there is in the csv
  number_of_rows <- nrow(dlucid)
  ## do the reference numbers, times 100 to get the decimal in place
  lucid_reference_numbers <- (column_sums / number_of_rows) * 100

  ##REFNUMS FOR PRECISE
  ## calculate the column sums
  column_sums <- apply(dprecise, 2, sum)
  ## how many lines there is in the csv
  number_of_rows <- nrow(dprecise)
  ## do the reference numbers, times 100 to get the decimal in place
  precise_reference_numbers <- (column_sums / number_of_rows) * 100

  change_rate <- (precise_reference_numbers - lucid_reference_numbers) + (lucid_reference_numbers - hardy_reference_numbers) + (hardy_reference_numbers - dapper_reference_numbers)
 
  ## Divide by 3 for the version changes
  change_rate <- change_rate / 3

  ## Combine the sums and reference numbers
  combined_data <- cbind(dapper_reference_numbers, hardy_reference_numbers, lucid_reference_numbers,    precise_reference_numbers, change_rate)

  combined_data

  ## sort the data into decreasing order
  sorted_data <- combined_data[order(change_rate, decreasing=TRUE),]

  write.table(sorted_data, file=rout, na="NA", dec=".", sep="\t")

}

calculate_refs("dapper-results/c.structs.1.results.csv", "hardy-results/c.structs.1.results.csv", "lucid-results/c.structs.1.results.csv", "precise-results/c.structs.1.results.csv", "structsDIFF.Rout")

calculate_refs("dapper-results/c.funcs.1.results.csv", "hardy-results/c.funcs.1.results.csv", "lucid-results/c.funcs.1.results.csv", "precise-results/c.funcs.1.results.csv", "funcsDIFF.Rout")

calculate_refs("dapper-results/c.consts.1.results.csv", "hardy-results/c.consts.1.results.csv", "lucid-results/c.consts.1.results.csv", "precise-results/c.consts.1.results.csv", "constsDIFF.Rout")

calculate_refs("dapper-results/c.ssl.1.results.csv", "hardy-results/c.ssl.1.results.csv", "lucid-results/c.ssl.1.results.csv", "precise-results/c.ssl.1.results.csv", "sslDIFF.Rout")

calculate_refs("dapper-results/c.scpt.1.results.csv", "hardy-results/c.scpt.1.results.csv", "lucid-results/c.scpt.1.results.csv", "precise-results/c.scpt.1.results.csv", "sctpDIFF.Rout")

