#!/bin/bash
#
#Expects folder structure
# apps-appname*-tarballs
#
# Goes over the folders unpacks the sources and patches them 
# identifies is this java python c ... and moves to correct folder.
#
#author: Samu Varjonen <samu.varjonen(a)cs.helsinki.fi>

basedir="apps"
targetdir="categorized-apps"
#mkdir -p ${targetdir}{/c,/cpp,/py,/py,/rb,/java,/other}
mkdir -p ${targetdir}{/c,/other}

echo -e "package;category;c;cpp;py;rb;java;php;sh;pl;html;txt;png;jpg;man;def;xml;js;tlc;m4;make-and-config;configuration;other;no extension"

#traverse the package folders
for f in `find $basedir -mindepth 1 -maxdepth 1`
do

	echo "Stepping into $f" 1>&2
	#traverse the files in package folder and unpack and patch
        for d in `find $f -mindepth 1 -maxdepth 1 | sort -r`
        do
		echo "unpacking $d" 1>&2
	        if [ ${d: -7} == "tar.bz2" ]; then
			tar -C $f -xjf $d
		elif [ ${d: -13} == "debian.tar.gz" ]; then
			tar -C $f -xzf $d 
			#patch in the subshell remove the folder debian.
			( cd $f ;  dpatch apply-all ; rm -rf debian/ )
	        elif [ ${d: -6} == "tar.gz" ]; then
			tar -C $f -xzf $d
	        elif [ ${d: -6} == "tar.xz" ]; then 
			tar -C $f -xJf $d
	        elif [ ${d: -7} == "diff.gz" ]; then 
			gunzip $d
			#some times the top most folder in the tar is different than what was expected.			
			archivename=`ls $f/*.orig.tar.gz`
			topmostfolderintar=`tar -tf $archivename | grep -o '^[^/]\+' | sort -u`
			whatitshouldbe=`ls $f/*.orig.tar.gz | rev | cut -d '.' -f4- | rev`
			#Run in subshell remove the gz and the path and patch and remove the diff 
                        mv $topmostfolderintar $whatitshouldbe
                        ( cd $f ; patch $whatitshouldbe -p1 < `echo $d | rev | cut -d '.' -f2- | rev | cut -d '/' -f3` ; rm *.diff )
		#else
		#	echo "SOME UNKNOWN EXTENSION $d"
		fi
 	done
	#traverse the same folder and count the extensions.
	cext=0
	cppext=0
	pyext=0
	rbext=0
	javaext=0 
	phpext=0
	shext=0
	plext=0
	htmlext=0
	txtext=0
	pngext=0
	jpgext=0
	manext=0
	confext=0
	defext=0
	xmlext=0
	jsext=0
	tlcext=0
	mfourext=0
	makeext=0
	cfgext=0
	otherext=0
	other=0
	biggest=""
	biggestnr=0

	
	for d in `find $f`
	do
		ending=`echo $d | rev | cut -d '.' -f1 | rev`
		ending=".$ending"
		#echo "checking $d ending *$ending*" 1>&2
		if [ $ending == "." ]; then 
			#sanity check for the ending as it might be zero length 
			echo zero length ending
			let other=other+1			
		elif [ $ending == ".c" ]; then
			let cext=cext+1
		elif [ $ending == ".cpp" ]; then 
			let cppext=cppext+1
		elif [ $ending == ".cc" ]; then
			let cppext=cppext+1
		elif [ $ending == ".py" ]; then
			let pyext=pyext+1
		elif [ $ending == ".rb" ]; then
			let rbext=rbext+1
		elif [ $ending == ".c" ]; then 
			let javaext=javaext+1
		elif [ $ending == ".php" ]; then 
			let phpext=phpext+1
		elif [ $ending == ".sh" ]; then
			let shext=shext+1
		elif [ $ending == ".pl" ]; then
			let plext=plext+1
		elif [ $ending == ".html" ]; then
			let htmlext=htmlext+1
		elif [ $ending == ".xhtml" ]; then
			let htmlext=htmlext+1 # I decided to count xhtml as html also
		elif [ $ending == ".txt" ]; then
			let txtext=txtext+1
		elif [ $ending == ".png" ]; then
			let pngext=pngext+1
		elif [ $ending == ".jpg" ]; then
			let jpgext=jpgext+1
		elif [ $ending == ".conf" ]; then
			let confext=confext+1
		elif [ $ending == ".def" ]; then
			let defext=defext+1
		elif [ $ending == ".xml" ]; then
			let xmlext=xmlext+1
		elif [ $ending == ".js" ]; then
			let jsext=jsext+1
		elif [ $ending == ".tcl" ]; then
			let tlcext=tlcext+1
		elif [ $ending == ".htcl" ]; then
			let tlcext=tlcext+1
		elif [ $ending == ".cfg" ]; then
			let cfgext=cfgext+1
		elif [ $ending == ".m4" ]; then
			let mfourext=mfourext+1
		elif [ $ending == ".ac" ]; then
			let makeext=makeext+1 
		elif [ $ending == ".in" ]; then
			let makeext=makeext+1
		elif [ $ending == ".am" ]; then
			let makeext=makeext+1
		elif [ $ending == ".1" ]; then 
			#man page
			let manext=manext+1
		elif [ $ending == ".2" ]; then 
			#man page
			let manext=manext+1 
		elif [ $ending == ".3" ]; then 
			#man page
			let manext=manext+1  
		elif [ $ending == ".4" ]; then 
			#man page
			let manext=manext+1 
		elif [ $ending == ".5" ]; then 
			#man page
			let manext=manext+1 
		elif [ $ending == ".6" ]; then 
			#man page
			let manext=manext+1 
		elif [ $ending == ".7" ]; then 
			#man page
			let manext=manext+1 
		elif [ $ending == ".8" ]; then 
			#man page
			let manext=manext+1 
		elif [ $ending == ".9" ]; then 
			#man page
			let manext=manext+1 
		else
			#check to see if there is any extension?
			#if there is no extension it must be a README etc.
			filename=$(basename "$d")
			extension="${filename##*.}"
			if [[ $filename =~ "." ]]; then 
				#skip c and cpp headers.
				if [ $extension != "h" ] && [ $extension != "hpp" ]; then
					let otherext=otherext+1
				fi
			else
				let other=other+1
			fi
		fi
	done

	if [[ $cext -gt $biggestnr ]]; then 
		biggestnr=$cext
		biggest="c"
	else #added to move all others to other folder
		biggest="other"
	fi
# commented out as we wanted to find all the packages with even one c file.
#	if [[ $cppext -gt $biggestnr ]]; then 
#		biggestnr=$cppext
#		biggest="cpp"
#	fi
#	if [[ $pyext -gt $biggestnr ]]; then 
#		biggestnr=$pyext
#		biggest="py"
#	fi
#	if [[ $rbext -gt $biggestnr ]]; then
#		biggestnr=$rbext
#		biggest="rb"
#	fi
#	if [[ $javaext -gt $biggestnr ]]; then
#		biggestnr=$javaext
#		biggest="java"
#	fi
#	if [[ $otherext -gt $biggestnr ]]; then 
#		biggestnr=$otherext
#		biggest="other"
#	fi
	
       #echo -e "package;category;c;cpp;py;rb;java;php;sh;pl;html;txt;png;jpg;man;def;xml;js;tlc;m4;make-and-config;configuration;other;no extension"
	echo -e "`echo $f | cut -d '/' -f2`;$biggest;$cext;$cppext;$pyext;$rbext;$javaext;$phpext;$shext;$plext;$htmlext;$txtext;$pngext;$jpgext;$manext;$defext;$xmlext;$jsext;$tlcext;$mfourext;$makeext;$cfgext;$otherext;$other"

	#move to correct folder
	mv -f `echo $d | rev | cut -d '/' -f2- | rev`/ ${targetdir}/${biggest}/

	echo "Stepping out $f" 1>&2

done

