## This R script that calculates the reference numbers 
## and outputs the relevant graphs to the pwd.
##
## Author: Samu Varjonen <samu.varjonen ( a ) hiit.fi>

## Function to do the graph
drawhist <- function(fdata, flabels, foutfile, fwidth, fheight){
  postscript(foutfile, horizontal = FALSE, onefile = FALSE, height = fheight, width = fwidth)  
  bp <- barplot(fdata, axes = FALSE, axisnames = FALSE, ylim = c(0,100))
  text(bp, par("usr")[3], labels = flabels, srt = 90, adj = c(1.1,1), cex = 0.7,xpd = TRUE)
  axis(2, at = c(0,10,20,30,40,50,60,70,80,90,100), cex = 0.7)
}

## Function to do the calculations per csv
calculate_refs <- function(filename, width, height, outname, rout, rout20){

  ## remove the app_name column
  data <- read.csv(filename)
  fdata <- data[, -1]
  ## remove the category column
  fdata <- fdata[, -1]
  
  ## calculate the column sums
  column_sums <- apply(fdata, 2, sum)

  ## how many lines there is in the csv
  number_of_rows <- nrow(fdata)

  ## do the reference numbers, times 100 to get the decimal in place
  reference_numbers <- (column_sums / number_of_rows) * 100
  
  ## Combine the sums and reference numbers
  combined_data <- cbind(column_sums, reference_numbers)

  ## sort the data into decreasing order
  sorted_data <- combined_data[order(column_sums, decreasing=TRUE),]

  write.table(sorted_data, file=rout, na="NA", dec=".", sep="\t")

  ## get the top 20 indicators
  top_20 <- head(sorted_data, 20)

  ## convert the results to dataframe
  complete = as.data.frame(top_20)

  ## Comment one out, the first prints table of the data and the second saves the graph
  write.table(complete, file=rout20, na="NA", dec=".", sep="\t")
  ##drawhist(complete$reference_number, row.names(complete), outname, width, height)

}

calculate_refs("saucy-results/c.structs.1.results.csv", 0, 4, "saucy_structs20.eps", "saucy_structs.Rout", "saucy_structs20.Rout")
calculate_refs("saucy-results/c.consts.1.results.csv", 0, 4, "saucy_consts20.eps", "saucy_consts.Rout", "saucy_consts20.Rout")
calculate_refs("saucy-results/c.funcs.1.results.csv", 0, 4, "saucy_funcs20.eps", "saucy_funcs.Rout", "saucy_funcs20.Rout")
calculate_refs("saucy-results/c.ssl.1.results.csv", 0, 0, "saucy_ssl20.eps", "saucy_ssl.Rout", "saucy_ssl20.Rout")
calculate_refs("saucy-results/c.scpt.1.results.csv", 0 ,4, "saucy_sctp20.eps", "saucy_sctp.Rout", "saucy_sctp20.Rout")
