#! /usr/bin/python

#######################
# This script reads all the app folders and then reads all the ".c" files.
# From the files comments are removed and line by line the lines are split 
# into words that are then compared to list of keywords.
# 
# input: path for the apps, path for the results, and the file extension c/cpp
#
# NOTE! Remember the trailing "/" in the paths in the command line args 
#
# Output is written in CSV format into files consts, funcs, structs, ssl, sctp
#
# author: Samu Varjonen <samu.varjonen (at) hiit.fi>

import ConfigParser
import os
import sys
import string 
import shlex
import time
import subprocess
from os import walk
from os.path import join, splitext
from pyparsing import cppStyleComment,dblQuotedString,Suppress

ext = string.strip(sys.argv[3])
#path_merged = "./" + string.strip(sys.argv[1]) + "lists/all.out.merge"
path_c_apps = "./" + string.strip(sys.argv[1]) + ext + "/"
path_results = "./" + string.strip(sys.argv[2]) 
 
c = ["." + ext]

print "\nStart phase 3.1\n"

t0 = time.time()

# input the configuration files to dictionaries
# it will be the {keyword, count} for the tuples.

dconsts = {}
dstructs = {}
dssl = {}
dsctp = {}
dfuncs = {}

def read_conf(filename, dictionary):  
    config = ConfigParser.ConfigParser()

    config.read(filename)

    for section in config.sections():
        #print section
        for option in config.options(section):
            #print " ", option, "=", config.get(section, option)
            dictionary[config.get(section, option).lower()] = 0
# END DEF

read_conf("consts.conf", dconsts)
read_conf("funcs.conf", dfuncs)
read_conf("structs.conf", dstructs)
read_conf("ssl.conf", dssl)
read_conf("sctp.conf", dsctp)

# Find out what is the shortest word. Usefull to check as
# we do not need to check all the items returned by the lexer.
# most of the items are "(", ")", etc any way.
def find_shortest_in(dict):
    shortest = 1000
    for k in dict:
        length = len(k)
        if length < shortest:
            shortest = length
    return shortest
# END DEF

#shortest_const = find_shortest_in(dconsts)
#shortest_func = find_shortest_in(dfuncs)
#shortest_struct = find_shortest_in(dstructs)
#shortest_ssl = find_shortest_in(dssl)
#shortest_sctp = find_shortest_in(dsctp)

def remove_comments(file):
    f=open(file, "r").read() 
    #f=unicode(f, "utf8")
    commentFilter = Suppress( cppStyleComment ).ignore( dblQuotedString )
    f_new= commentFilter.transformString(f)
    return f_new
# END DEF

def string_lexical(file):
    lexer = shlex.shlex(file)
    temp = []
    try:
        for token in lexer:
            temp.append(token)
    except:
        return temp	
    return temp
# END DEF

#THIS WHOLE THING IS FOR THE OLD DOWNLOAD SCRIPTS
# Fill the dictionary with the information of all the applications.
# This is used to find out to what category the application belongs to 
# so the correct download URLs can be constructed.
#D = {}
#f = open(path_merged)
#while 1:
#    line = f.readline()
#    if not line:
#        break
#    line = string.strip(line)  
#    pieces = line.split(",")
#    D[pieces[0]] = line

# This function digs through the dictionary above in order to find the correct category
#def find_category(name):
#    the_category = "Something_went_wrong"
#    for k, v in D.iteritems():
#        if k == "\"" + name +"\"":
#            tmp_category = v.split(",")
#            the_category = tmp_category[4]
#            the_category = the_category[1:len(the_category)-1]
#    return the_category
# end def 

# Do the stats !Not in use anymore!
#def is_the(word, in_dict, min):
#    if len(word) < min:
#        #print "Do not bother"
#        return
#    #print "Trying it"
#    lword = word.lower()
#    if in_dict.has_key(lword):
#        #print "FOUND YOU \"" + word + "\"" 
#        in_dict[lword] = in_dict[lword] + 1
#        #print lword + " " + str(in_dict[lword])
# END DEF

# Print out the title lines for the result files
def title_lines(f, dict):
    f.write("\"app_name\"")
    f.write(",\"category\"")
    for k in dict:
        f.write(",\"" + k + "\"")
    f.write("\n")
# END DEF

# counts
fr_funcs = open(path_results + ext + ".funcs.results.csv", 'w')
fr_consts = open(path_results + ext + ".consts.results.csv", 'w')
fr_structs = open(path_results + ext + ".structs.results.csv", 'w')
fr_ssl = open(path_results + ext + ".ssl.results.csv", 'w')
fr_sctp = open(path_results + ext + ".scpt.results.csv", 'w')
# reference ratio "1's"
fr1_funcs = open(path_results + ext + ".funcs.1.results.csv", 'w')
fr1_consts = open(path_results + ext + ".consts.1.results.csv", 'w')
fr1_structs = open(path_results + ext + ".structs.1.results.csv", 'w')
fr1_ssl = open(path_results + ext + ".ssl.1.results.csv", 'w')
fr1_sctp = open(path_results + ext + ".scpt.1.results.csv", 'w')

title_lines(fr_funcs , dfuncs)
title_lines(fr_structs , dstructs)
title_lines(fr_consts , dconsts)
title_lines(fr_ssl , dssl)
title_lines(fr_sctp , dsctp)

title_lines(fr1_funcs , dfuncs)
title_lines(fr1_structs , dstructs)
title_lines(fr1_consts , dconsts)
title_lines(fr1_ssl , dssl)
title_lines(fr1_sctp , dsctp)

# clear the results for the next app
def clear_sums(dict):
    for k in dict:
        dict[k] = 0
# END DEF

# Change the larger than 0 counts to 1
def onesies(dict):
    for k in dict:
        if dict[k] > 0:
            dict[k] = 1
# END DEF

# print function for the results line
def apps_result_line(f, dict, app_name):
    f.write("\"" + app_name + "\"")
#   f.write(",\"" + find_category(app_name) + "\"")
    f.write(",\"" + "categoryNA" + "\"")
    for k in dict:
        f.write("," + str(dict[k]))
    f.write("\n")
# END DEF
 
def count_apps():
    count = 0 
    for tdirname in os.listdir(path_c_apps):
        count = count + 1
    return count
# END DEF

# add the file count to total count
def apps_add_file_count(dict, file):
    for k in dict:
        dict[k] = dict.get(k, 0) + file.get(k, 0)
# END DEF

# Start reading the app data...
app_num_now = 0
app_count = count_apps()
for tdirname in os.listdir(path_c_apps):
    if os.path.isdir(os.path.join(path_c_apps, tdirname)):
        app_num_now = app_num_now + 1
        print "* " + tdirname + " (" + str(app_num_now) + "/" + str(app_count) + \
              ")***************************"
        path_this_far = os.path.join(path_c_apps, tdirname)
        for dirname, dirnames, filenames in os.walk(path_this_far):
            for filename in filenames:                
                extension = os.path.splitext(filename)[1]
                if extension in c:
                    print "++ " + filename
                    filedict = {}
                    myprocess = subprocess.Popen(["./command.sh", join(dirname, filename)], stdout=subprocess.PIPE)
                    (sout,serr) = myprocess.communicate() 
    	            for line in sout.split('\n'):
                        if len(line) > 0:
	                    pieces = line.split('\t')
		            filedict[pieces[0]] = filedict.get(pieces[0], 0) + int(pieces[1])
    	            myprocess.wait()               # We wait for process to finish
                    #print filedict
                    apps_add_file_count(dfuncs, filedict) 
                    apps_add_file_count(dconsts, filedict)
                    apps_add_file_count(dstructs, filedict)
                    apps_add_file_count(dssl, filedict)
                    apps_add_file_count(dsctp, filedict)
    # Print out the results for this app
    # first the count
    apps_result_line(fr_funcs, dfuncs, tdirname)
    apps_result_line(fr_consts, dconsts, tdirname)                    
    apps_result_line(fr_structs, dstructs, tdirname)                    
    apps_result_line(fr_ssl, dssl, tdirname)                    
    apps_result_line(fr_sctp, dsctp, tdirname)                    

    # then the onesies
    onesies(dfuncs)
    onesies(dstructs)
    onesies(dconsts)
    onesies(dssl)
    onesies(dsctp)
    apps_result_line(fr1_funcs, dfuncs, tdirname)
    apps_result_line(fr1_consts, dconsts, tdirname)                    
    apps_result_line(fr1_structs, dstructs, tdirname)                    
    apps_result_line(fr1_ssl, dssl, tdirname)                    
    apps_result_line(fr1_sctp, dsctp, tdirname)

    # clear the dictionaries for the next round
    clear_sums(dfuncs)
    clear_sums(dconsts)
    clear_sums(dstructs)
    clear_sums(dssl)
    clear_sums(dsctp)

print "\nStop phase 3.1\n"
print "Time it took " + str(time.time() -t0) + " seconds\n"
